;***************************************************************************
; 
; File Name		:'Main.asm"
; Title			:AFSK Modulator 
; Date			:2002.03.22.
; Version		:1.0.0
; Support telephone	:+36-70-333-4034
; Support fax		:+36-30-9809-372
; Support Email		:info@vfx.hu
; Target MCU		:AT90S2313
; 
;***************************************************************************
;	D E S C R I P T I O N
;
;  PortB: FSK analog 8 bit data out
;  PD6:   FSK Input
;
;***************************************************************************
;	M O D I F I C A T I O N   H I S T O R Y 
;
;
;       rev.      date      who  why
;	----	----------  ---	--------------------------------------------
;	0.01	2002.03.22  VFX	Creation
;
;
;***************************************************************************
;Hardware
;***************************************************************************
;*
;*	Kvarc: f=4.000 MHz (T=125.00ns)
;*
;***************************************************************************
;
;
;
.include "2313def.inc"


;***************************************************************************
;* Const Def

;* 32 minta/cyles  11.25fok f=4.000MHz
;* 1200Hz  -> 38400Hz -- 104 -> 1202Hz
;* 2200Hz  -> 70400Hz --  57 -> 2193Hz

.EQU	LowFrekv = (256-77)
.EQU	HiFrekv = (256-38)		


;**************************************************************************
;* Hardware Def.


.EQU	FSKPort = PORTB
.EQU	FSKPortDIR = DDRB




;***************************************************************************
;**** VARIABLES
.DSEG

RAMTable:	.byte 32	;Sinus table	

;***************************************************************************
.ESEG
		.DB 	"AFSK Modulator [2002.03.22.]"
		.DB	"Latest Upd.: 2002.03.24."
		.DB	"Rev:1.0.0 "
		.DB	"info@vfx.hu"


;*****************************************************************************
;****		I N T E R R U P T S
;****
;*****************************************************************************
.CSEG
		rjmp RESET		;Reset Handle
		rjmp EXT_INTO		;IRQ0 Handle
		rjmp EXT_INT1		;IRQ1 Handle
		rjmp TIM_CAPT1		;Timer1 capture Handle
		rjmp TIM_COMP1		;Timer1 compare Handle
		rjmp TIM_OVF1		;Timer1 overflow Handle
		rjmp TIM_OVF0		;Timer0 overflow Handle
		rjmp UART_RXC		;UART RX Complete Handle
		rjmp UART_DRE		;UDR Empty Handle
		rjmp UART_TXC		;UART TX Complete Handle
		rjmp ANA_COMP		;Analog Comparator Handle

;********************************
;* RESET 
;********************************
RESET:		CLI			;GLOBAL INTERRUP DIS.	

		ldi R16, low(RAMEND)
		out SPL,R16

		LDI	R16,0b00101010	;Sleep enable, Idle mode,INT0, INT1 edge sens.
		OUT	MCUCR,R16	;7. = 1 SRE External SRAM Enabled
					;6. = 1 SRW Ext. SRAM Wait State Enabled
					;5. = 1 Sleep Enable
					;4. = 0 SLEEP => Idle Mode
					;     1 SLEEP => Power Down Mode
					;3:2 = Inerrupt Sense Control for INT1
					; 00 -> Low Level INT1
					; 01 -> Reserved
					; 10 -> Falling Edge INT1
					; 11 -> Rising Edge INT1
					;1:0 = Inerrupt Sense Control for INT0
					; 00 -> Low Level INT0
					; 01 -> Reserved
					; 10 -> Falling Edge INT0
					; 11 -> Rising Edge INT0



		LDI	R16,0b00000000	;INT0, INT1 Disabled
		OUT	GIMSK,R16	;7. = 1 EXT INT1 ACTIVE
					;6. = 1 EXT INT0 ACTIVE



		LDI	R16,0b00000010
		OUT	TIMSK,R16 	;7. = 1 TC1 OWRF EN.  *
					;6. = 1 TC1 COMP MATCH EN.
					;3. = 1 TC1 INP CAPTURE EN.
					;1. = 1 TC0 OWRF EN.   *

;***************************************************************************
;**  Timer0 Init		*****
;**
InitTimer0:	LDI	R20,LowFrekv
		OUT	TCNT0,R16	;Alapbol 1200Hz
		LDI	R16,0b00000001
		OUT	TCCR0,R16	;TC0 Control Register
					;2:0 Clock Select
					; 000 -> Stop
					; 001 -> CK
					; 010 -> CK/8
					; 011 -> CK/64
					; 100 -> CK/256
					; 101 -> CK/1024
					; 110 -> External pin T0 falling edge
					; 111 -> External pin T0 risinrg edge

					;megszkitas engedelyezese elott
					;toroljuk az INT flagaket
					;ami eddig jott volna megszakitas az torlodik

                                        
		ldi	ZL,Low(ROMTable)
		ldi	ZH,High(ROMTable)
		ldi	R16,32		;32 adat lesz egy sinus

		ldi	XL,Low(RAMTable)
		ldi	XH,High(RAMTable)

CopyRomToRAM:	lpm
		adiw	ZL,1
		st	X+,R0
		dec	R16
		brne	CopyRomToRAM

		ldi	XL,Low(RAMTable)
		ldi	XH,High(RAMTable)


		cbi	DDRD,PD6	;PD6 input
		sbi	PORTD,PD6	;Pull-up


;		sbi	DDRD,PD5	;output
;		sbi	PORTD,PD5

                                    

		ldi	R16,0
		out	FSKPort,R16	;kimeneti bitek = 0
		ldi	R16,255
		out	FSKPortDIR,R16	;minden bit output

		clr	R21		;Flag lesz

		SEI			;SET GLOBAL INT EN.

		RJMP	Main


;*******************
EXT_INTO:	RETI
EXT_INT1:	RETI
TIM_CAPT1:	RETI
TIM_COMP1:	RETI
TIM_OVF1:	RETI

;********************
;* Timer0 Int
;* 
TIM_OVF0:	OUT	TCNT0,R20
		ser	R21		;itt az ideje uj adatot kuldeni
		RETI

;*****************
;* UART RX COMPLET 
UART_RXC:	RETI

;*******************
UART_DRE:	RETI

;******************
;* UART TX COMPLET 
UART_TXC:	RETI

ANA_COMP:	RETI


ROMTable:
		.db	127,152
		.db	176,198
		.db	217,233
		.db	245,252
		.db	255,252
		.db	245,233
		.db	217,198
		.db	176,152
		.db	127,102
		.db	78,56
		.db	36,20
		.db	9,2
		.db	0,2
		.db	9,20
		.db	36,56
		.db	78,102



;***********************************************************************
;***********************************************************************
;********     M     A      I      N				********
;********							********
;***********************************************************************
Main:		
		
		ldi	R22,HiFrekv
		sbic	PIND,PD6   	;ha inp=L a kovetkezo kimarad
		 ldi	R22,LowFrekv
		mov	R20,R22		;csak hogy ne prellergjen

		sleep

		ld	R0,X+
		andi	XL,31		;csak 0...31-ig cimzunk
					;majd atfordul
		out	FSKPort,R0	;new data a porton

		rjmp	Main

